@@ -183,9 +183,9 @@ std::string thcs_get_trans(int from_cs, int to_cs) {
 
 void thcs_add_cs_trans_single(const char * from_cs, const char * to_cs, const char * trans) {
 	int fcs = thcs_parse(from_cs);
-	if (fcs == TTCS_UNKNOWN) ththrow("uknown coordinate system -- {}", from_cs);
+	if (fcs == TTCS_UNKNOWN) ththrow("unknown coordinate system -- {}", from_cs);
 	int tcs = thcs_parse(to_cs);
-	if (tcs == TTCS_UNKNOWN) ththrow("uknown coordinate system -- {}", to_cs);
+	if (tcs == TTCS_UNKNOWN) ththrow("unknown coordinate system -- {}", to_cs);
 //	if (strlen(trans) == 0) ththrow("empty transformation specification");
 	thcs_transformations[thcstrans(fcs, tcs)] = thdb.strstore(trans, true);
 }
