@@ -44,14 +44,14 @@
 #include <list>
 #include "thmapstat.h"
 
-void thdb2d::insert_basic_maps(thdb2dxm * fmap, thmap * map, int mode, int level) {
+void thdb2d::insert_basic_maps(thdb2dxm * fmap, thmap * map, int mode, int level, thdb2dmi_shift shift) {
   thdb2dxs * xs, * txs = NULL;
   thdb2dmi * mi;
   bool found = false;
   if (map->is_basic) {
     xs = fmap->first_bm;
     while ((!found) && (xs != NULL)) {
-      if (xs->bm->id == map->id) {
+      if ((xs->bm->id == map->id) && (xs->m_shift == shift)) {
         found = true;
         txs = xs;
       }
@@ -62,6 +62,7 @@ void thdb2d::insert_basic_maps(thdb2dxm * fmap, thmap * map, int mode, int level
       txs = xs;
       xs->bm = map;
       xs->fmap = fmap;
+      xs->m_shift = shift;
       if (fmap->first_bm == NULL) {
         fmap->first_bm = xs;
         fmap->last_bm = xs;
@@ -72,14 +73,36 @@ void thdb2d::insert_basic_maps(thdb2dxm * fmap, thmap * map, int mode, int level
         fmap->last_bm = xs;
       }
     }
+
+
+    thdb2dxs_target_item tmp_target(map->projection_id, map->id, shift);
+    thdb2dxs_target_map::iterator ti;
+    ti = this->m_target_map.find(tmp_target);
+    if (ti == this->m_target_map.end()) {
+      this->m_target_map[tmp_target] = txs;
+      txs->m_target = txs;
+    } else {
+      txs->m_target = ti->second;
+    }
+      
+    
+
 //    if (txs->mode < mode) {
     if (txs->mode != TT_MAPITEM_NORMAL) {
       txs->mode = mode;
-      mi = map->first_item;
+      txs->m_shift = shift;
+//      mi = map->first_item;
 //      if (map->selection_mode < mode) {
-      if (map->selection_mode != TT_MAPITEM_NORMAL) {
-        map->selection_mode = mode;
-        map->selection_xs = txs;
+
+
+//      if (map->selection_mode != TT_MAPITEM_NORMAL) {
+//        map->selection_mode = mode;
+//        map->selection_xs = txs;
+//      }
+
+      if (txs->m_target->mode != TT_MAPITEM_NORMAL) {
+        txs->m_target = txs;
+        this->m_target_map[tmp_target] = txs;
       }
 //      while (mi != NULL) {
 //        if ((mi->type == TT_MAPITEM_NORMAL) && (((thscrap*)(mi->object))->selection_mode < mode)) {
@@ -96,11 +119,15 @@ void thdb2d::insert_basic_maps(thdb2dxm * fmap, thmap * map, int mode, int level
   while (mi != NULL) {
     if (mode == TT_MAPITEM_NORMAL) {
       if (((mi->type != TT_MAPITEM_NORMAL) || (!map->is_basic)) && 
-          ((mi->type == TT_MAPITEM_NORMAL) || (level == 0)))
-        this->insert_basic_maps(fmap,(thmap *)mi->object,mi->type,level+1);
+        ((mi->type == TT_MAPITEM_NORMAL) || (level == 0))) {
+          this->insert_basic_maps(fmap,(thmap *)mi->object,mi->type,level+1, shift.add(mi->m_shift));
+      }
+      if ((mi->type == TT_MAPITEM_NORMAL) && (mi->m_shift.is_active()) && (mi->m_shift.m_preview != TT_MAPITEM_NONE)) {
+        this->insert_basic_maps(fmap,(thmap *)mi->object, mi->m_shift.m_preview, level+1, shift);
+      }
     } else {
       if ((mi->type == TT_MAPITEM_NORMAL) && (!map->is_basic))
-        this->insert_basic_maps(fmap,(thmap *)mi->object,mode,level+1);
+        this->insert_basic_maps(fmap,(thmap *)mi->object,mode,level+1, shift);
     }
     mi = mi->next_item;
   }
@@ -126,12 +153,14 @@ int thdb2d_compxm(const void * ee1, const void * ee2)
 
 int thdb2d_compscrap(const void * ee1, const void * ee2)
 {
-  thscrap * e1 = (thscrap *) ee1, * e2 = (thscrap *) ee2;
+  thscrap * e1, * e2;
+  e1 = *((thscrap **) ee1);
+  e2 = *((thscrap **) ee2);
   if (thisnan(e1->z))
-    return -1;
-  if (thisnan(e2->z))
     return 1;
-  if (e1->z < e2->z) {
+  if (thisnan(e2->z))
+    return -1;
+  if (e1->z > e2->z) {
     return -1;
   } else if (e1->z == e2->z)
     return 0;
@@ -140,7 +169,7 @@ int thdb2d_compscrap(const void * ee1, const void * ee2)
 }
 
 
-thdb2dxm * thdb2d::insert_maps(thdb2dxm * selection,thdb2dxm * insert_after,thmap * map, 
+thdb2dxm * thdb2d::insert_maps(thdb2dxm * selection,thdb2dxm * insert_after, thmap * map, 
     unsigned long selection_level, int level, int title_level, int map_level) {
   thdb2dxm * cxm = NULL;
   thdb2dmi * mi;
@@ -172,10 +201,10 @@ thdb2dxm * thdb2d::insert_maps(thdb2dxm * selection,thdb2dxm * insert_after,thma
     while (mi != NULL) {
       if (mi->type == TT_MAPITEM_NORMAL) {
         if (cxm == NULL)
-          selection = this->insert_maps(selection,insert_after,(thmap *) mi->object, 
+          selection = this->insert_maps(selection,insert_after,(thmap *) mi->object,
             selection_level, level+1, title_level, map_level);
         else 
-          selection = this->insert_maps(selection,cxm,(thmap *) mi->object, 
+          selection = this->insert_maps(selection,cxm,(thmap *) mi->object,
             selection_level, level+1, title_level, map_level);
       }
       mi = mi->next_item;
@@ -279,7 +308,7 @@ thdb2dxm * thdb2d::select_projection(thdb2dprj * prj)
   thselector_list::iterator ii = thcfg.selector.data.begin();
   cxm = selection;
   while (ii != thcfg.selector.data.end()) {
-    if ((!ii->unselect) && (ii->optr != NULL) && 
+    if ((!ii->unselect) && (ii->optr != NULL) && thcfg.use_maps &&
         (ii->optr->get_class_id() == TT_MAP_CMD) &&
         (((thmap*)(ii->optr))->projection_id == prj->id)) {
       selection = this->insert_maps(selection,cxm,(thmap*)(ii->optr),
@@ -287,28 +316,38 @@ thdb2dxm * thdb2d::select_projection(thdb2dprj * prj)
         ((ii->map_level >= 0) && (ii->chapter_level > ii->map_level) ? ii->map_level : ii->chapter_level),ii->map_level);
       prj->stat.scanmap((thmap*)(ii->optr));  
       prj->stat.addstat(&(((thmap*)(ii->optr))->stat));
+      // set selection color
+      lxm = selection;
+      while (lxm != NULL) {
+    	if ((lxm->selection_level == ii->number) && (ii->m_color.defined))
+    		lxm->selection_color = ii->m_color;
+        lxm = lxm->next_item;
+      }
       while ((cxm != NULL) && (cxm->next_item != NULL))
         cxm = cxm->next_item;
     }
     ii++;
   }
   
+  lxm = NULL;
   // no map selected, let's add all basic maps
   if (selection == NULL) {
     nmaps = 0;
     obi = this->db->object_list.begin();
     while (obi != this->db->object_list.end()) {
       if (((*obi)->get_class_id() == TT_MAP_CMD) &&
-          (((thmap*)(*obi))->projection_id == prj->id) &&
-          (((thmap*)(*obi))->is_basic) &&
-          (((thmap*)(*obi))->fsptr != NULL) &&
-          (((thmap*)(*obi))->fsptr->is_selected())) {
-        prj->stat.scanmap((thmap*)(*obi));  
-        prj->stat.addstat(&(((thmap*)(*obi))->stat));
+		  thcfg.use_maps &&
+          (((thmap*)(*obi).get())->projection_id == prj->id) &&
+          (((thmap*)(*obi).get())->is_basic) &&
+          (((thmap*)(*obi).get())->fsptr != NULL) &&
+          (((thmap*)(*obi).get())->fsptr->is_selected())) {
+        prj->stat.scanmap((thmap*)(*obi).get());  
+        prj->stat.addstat(&(((thmap*)(*obi).get())->stat));
         cxm = this->insert_xm();
+        cxm->selection_color = ((thmap*)(*obi).get())->fsptr->selected_color;
         cxm->title = false;
         cxm->expand = true;
-        cxm->map = (thmap*)(*obi);
+        cxm->map = (thmap*)(*obi).get();
         cxm->map->calc_z();
         nmaps++;
         if (selection == NULL) {
@@ -321,39 +360,106 @@ thdb2dxm * thdb2d::select_projection(thdb2dprj * prj)
         }
       }
       obi++;
-    }    
-    
+    } 
+
+    // add centerline to map, even some basic maps exists
+    //if (nmaps > 0) {
+
+    //  thscrap * scrapp;
+    //  thmap * mapp;
+    //  thdb2dmi * xcitem;
+
+    //  scrapp = new thscrap;
+    //  scrapp->centerline_io = true;
+    //  scrapp->fsptr = NULL;
+    //  scrapp->db = &(thdb);
+    //  scrapp->proj = prj;
+    //  thdb.object_list.push_back(scrapp);
+
+    //  mapp = new thmap;
+    //  mapp->db = &(thdb);
+    //  mapp->fsptr = NULL;
+    //  thdb.object_list.push_back(mapp);
+
+    //  xcitem = thdb.db2d.insert_map_item();
+    //  xcitem->itm_level = mapp->last_level;
+    //  xcitem->source = thdb.csrc;
+    //  xcitem->psurvey = NULL;
+    //  xcitem->type = TT_MAPITEM_NORMAL;
+    //  xcitem->object = scrapp;
+    //  mapp->first_item = xcitem;
+    //  mapp->last_item = xcitem;
+
+    //  cxm = this->insert_xm();
+    //  cxm->title = false;
+    //  cxm->expand = true;
+    //  cxm->map = mapp;
+    //  prj->stat.scanmap(mapp);  
+    //  prj->stat.addstat(&(mapp->stat));
+
+    //  cxm->next_item = selection;
+    //  selection->prev_item = cxm;
+    //  selection = cxm;
+    //  nmaps++;
+
+    //}
 
     if ((selection == NULL) && (
         (prj->type == TT_2DPROJ_PLAN) || (prj->type == TT_2DPROJ_EXTEND) ||
-        (prj->type == TT_2DPROJ_ELEV)) &&
-        (thdb.db1d.lsid > 0)) {
+        (prj->type == TT_2DPROJ_ELEV))) {
       // podme vytvorit jednu mapu a vlozit do nej
       // vsetky surveye
       thscrap * scrapp;
       thmap * mapp;
       thdb2dmi * xcitem;
 
-      scrapp = new thscrap;
-      scrapp->centerline_io = true;
-      scrapp->fsptr = NULL;
-      scrapp->db = &(thdb);
-      scrapp->proj = prj;
-      thdb.object_list.push_back(scrapp);
+      // najprv spocitame scrapy
+      nscraps = 0;
+      obi = this->db->object_list.begin();
+      while (obi != this->db->object_list.end()) {
+        if (((*obi)->get_class_id() == TT_SCRAP_CMD) &&
+            (((thscrap*)(*obi).get())->proj->id == prj->id) &&
+            (((thscrap*)(*obi).get())->fsptr != NULL) &&
+            (((thscrap*)(*obi).get())->fsptr->is_selected())) {
+          nscraps++;
+        }
+        obi++;
+      }  
 
-      mapp = new thmap;
-      mapp->db = &(thdb);
-      mapp->fsptr = NULL;
-      thdb.object_list.push_back(mapp);
+      {
+        auto unique_mapp = std::make_unique<thmap>();
+        mapp = unique_mapp.get();
+        mapp->id = ++this->db->objid;
+        mapp->projection_id = prj->id;
+        mapp->db = &(thdb);
+        mapp->fsptr = NULL;
+        thdb.object_list.push_back(std::move(unique_mapp));
+      }
 
-      xcitem = thdb.db2d.insert_map_item();
-      xcitem->itm_level = mapp->last_level;
-      xcitem->source = thdb.csrc;
-      xcitem->psurvey = NULL;
-      xcitem->type = TT_MAPITEM_NORMAL;
-      xcitem->object = scrapp;
-      mapp->first_item = xcitem;
-      mapp->last_item = xcitem;
+      // ak nemame ani jeden scrap, vytvorime mapu z centerlajnu
+      if (nscraps == 0) {
+
+        {
+          auto unique_scrapp = std::make_unique<thscrap>();
+          scrapp = unique_scrapp.get();
+          scrapp->centerline_io = true;
+          scrapp->fsptr = NULL;
+          scrapp->db = &(thdb);
+          scrapp->proj = prj;
+          thdb.object_list.push_back(std::move(unique_scrapp));
+        }
+
+        xcitem = thdb.db2d.insert_map_item();
+        xcitem->itm_level = mapp->last_level;
+        xcitem->source = thdb.csrc;
+        xcitem->psurvey = NULL;
+        xcitem->type = TT_MAPITEM_NORMAL;
+        xcitem->object = scrapp;
+
+        mapp->first_item = xcitem;
+        mapp->last_item = xcitem;
+
+      }
       
       // do tej mapy povkladame vsetky scrapy, ktore
       // v danej projekcii mame zoradene podla Ztka a kazde
@@ -362,34 +468,39 @@ thdb2dxm * thdb2d::select_projection(thdb2dprj * prj)
       obi = this->db->object_list.begin();
       while (obi != this->db->object_list.end()) {
         if (((*obi)->get_class_id() == TT_SCRAP_CMD) &&
-            (((thscrap*)(*obi))->proj->id == prj->id) &&
-            (((thscrap*)(*obi))->fsptr != NULL) &&
-            (((thscrap*)(*obi))->fsptr->is_selected())) {
-          xcitem = thdb.db2d.insert_map_item();
-          mapp->last_item->next_item = xcitem;
-          xcitem->prev_item = mapp->last_item;
-          mapp->last_item = xcitem;
+            (((thscrap*)(*obi).get())->proj->id == prj->id) &&
+            (((thscrap*)(*obi).get())->fsptr != NULL) &&
+            (((thscrap*)(*obi).get())->fsptr->is_selected())) {
+          xcitem = thdb.db2d.insert_map_item();          
+          if (mapp->first_item == NULL) {
+            mapp->first_item = xcitem;
+            mapp->last_item = xcitem;
+          } else {
+            mapp->last_item->next_item = xcitem;
+            xcitem->prev_item = mapp->last_item;
+            mapp->last_item = xcitem;
+          }
           xcitem->itm_level = ++mapp->last_level;
           xcitem->source = thdb.csrc;
           xcitem->psurvey = NULL;
           xcitem->type = TT_MAPITEM_NORMAL;
-          xcitem->object = (thscrap*)(*obi);
+          xcitem->object = (thscrap*)(*obi).get();
           nscraps++;
         }
         obi++;
       }  
       
-      if (nscraps > 0) {
+      if (nscraps > 1) {
 
         // zoradime scrapy podla z-ka
         thscrap ** sss = new thscrap* [nscraps];
-        xcitem = mapp->first_item->next_item;
+        xcitem = mapp->first_item; //->next_item;
         for(iscr = 0; iscr < nscraps; iscr++) {
           sss[iscr] = (thscrap *) xcitem->object;
           xcitem = xcitem->next_item;
         }
         qsort(sss, nscraps, sizeof(thscrap*), thdb2d_compscrap);
-        xcitem = mapp->first_item->next_item;
+        xcitem = mapp->first_item; //->next_item;
         for(iscr = 0; iscr < nscraps; iscr++) {
           xcitem->object = sss[iscr];
           xcitem = xcitem->next_item;
@@ -446,8 +557,8 @@ thdb2dxm * thdb2d::select_projection(thdb2dprj * prj)
     cxm = selection;
     while (cxm != NULL) {
       if (cxm->expand) {
-        this->insert_basic_maps(cxm,cxm->map,TT_MAPITEM_NORMAL,0);
         cxm->output_number = ++on;
+        this->insert_basic_maps(cxm,cxm->map,TT_MAPITEM_NORMAL,0);
       }
       cxm = cxm->next_item;
     }
@@ -460,15 +571,28 @@ thdb2dxm * thdb2d::select_projection(thdb2dprj * prj)
       while (pcxs != NULL) {
         // ak je fmap zakladna, tak OK a PON = ON
         // inak PON = ++on
-        if (pcxs->bm->selection_xs->preview_output_number == 0) {
-          if ((pcxs->bm->selection_mode == TT_MAPITEM_NORMAL) &&
-              pcxs->bm->selection_xs->fmap->map->is_basic) 
-            pcxs->bm->selection_xs->preview_output_number =
-                pcxs->bm->selection_xs->fmap->output_number;
+
+        thdb2dxs_target_item tmp_target(pcxs->bm->projection_id, pcxs->bm->id, pcxs->m_shift);
+        pcxs->m_target = this->m_target_map[tmp_target];
+
+        if (pcxs->m_target->preview_output_number == 0) {
+          if ((pcxs->m_target->mode == TT_MAPITEM_NORMAL)
+              && pcxs->m_target->fmap->map->is_basic)
+            pcxs->m_target->preview_output_number = pcxs->m_target->fmap->output_number;
           else
-            pcxs->bm->selection_xs->preview_output_number = ++on;
-//            pcxs->bm->selection_xs->preview_output_number = 0;
+            pcxs->m_target->preview_output_number = ++on;
         }
+
+
+//        if (pcxs->bm->selection_xs->preview_output_number == 0) {
+//          if ((pcxs->bm->selection_mode == TT_MAPITEM_NORMAL) &&
+//              pcxs->bm->selection_xs->fmap->map->is_basic) 
+//            pcxs->bm->selection_xs->preview_output_number =
+//                pcxs->bm->selection_xs->fmap->output_number;
+//          else
+//            pcxs->bm->selection_xs->preview_output_number = ++on;
+//        }       
+
         pcxs = pcxs->next_item;
       }
       cxm = cxm->next_item;
@@ -479,11 +603,12 @@ thdb2dxm * thdb2d::select_projection(thdb2dprj * prj)
 }
 
 void thdb2d::reset_selection() {
+  this->m_target_map.clear();
   thdb_object_list_type::iterator obi = this->db->object_list.begin();
   while (obi != this->db->object_list.end()) {
     switch ((*obi)->get_class_id()) {
       case TT_MAP_CMD:
-        ((thmap *)(*obi))->selection_mode = TT_MAPITEM_UNKNOWN;
+        ((thmap *)(*obi).get())->selection_mode = TT_MAPITEM_UNKNOWN;
         break;
 //      case TT_SCRAP_CMD:
 //        ((thscrap *)(*obi))->selection_mode = TT_MAPITEM_UNKNOWN;
@@ -494,13 +619,13 @@ void thdb2d::reset_selection() {
   }
 }
 
-char * thdb2dscan_survey_title(thsurvey * fptr, long & min) {
+const char * thdb2dscan_survey_title(thsurvey * fptr, long & min) {
 
   long newmin = 0, tmpmin;
   if (fptr == NULL) {
     return "";
   }
-  char * newname = fptr->title, * tmpname;
+  const char * newname = fptr->title, * tmpname;
   if (strlen(newname) == 0)
     newname = fptr->name;
   thdataobject * o;
@@ -546,13 +671,58 @@ char * thdb2dscan_survey_title(thsurvey * fptr, long & min) {
   return newname;
 }
 
-char * thdb2d::get_projection_title(thdb2dprj * prj) {
+
+double thdb2d::get_projection_entrance_altitude(thdb2dprj * prj) {
+	  thobjectname entrance;
+	  thselector_list::iterator ii = thcfg.selector.data.begin();
+	  while (ii != thcfg.selector.data.end()) {
+	    if ((!ii->unselect) && (ii->optr != NULL) &&
+	        (ii->optr->get_class_id() == TT_MAP_CMD) &&
+	        (((thmap*)(ii->optr))->projection_id == prj->id)) {
+	      if (entrance.is_empty()) {
+	    	  entrance = ((thmap*)(ii->optr))->fsptr->get_entrance();
+	      }
+	    }
+	    ii++;
+	  }
+	  thsurvey * cs;
+	  if (entrance.is_empty()) {
+		  auto obi = this->db->object_list.begin();
+		  while (obi != this->db->object_list.end()) {
+			  if ((*obi)->get_class_id() == TT_SURVEY_CMD) {
+			        cs = (thsurvey *)(*obi).get();
+			        if (entrance.is_empty()) {
+				        if (cs->is_selected()) entrance = cs->get_entrance();
+			        }
+			  }
+			  obi++;
+		  }
+	  }
+	  if (entrance.is_empty()) {
+		  auto obi = this->db->object_list.begin();
+		  while (obi != this->db->object_list.end()) {
+			  if ((*obi)->get_class_id() == TT_SURVEY_CMD) {
+			        cs = (thsurvey *)(*obi).get();
+			        if (entrance.is_empty()) {
+				        entrance = cs->get_entrance();
+			        }
+			  }
+			  obi++;
+		  }
+	  }
+	  if (entrance.is_empty()) return 0;
+	  return this->db->db1d.station_vec[entrance.id-1].z;
+
+}
+
+
+const char * thdb2d::get_projection_title(thdb2dprj * prj) {
 
   // krok cislo jedna - prejde celu selection - ak najde jednu oznacenu
   // mapu - tak zoberie jej title //alebo name
   long dum;
   
-  char * rv = NULL;
+  const char * rv = NULL;
   unsigned long nmaps = 0;
   thselector_list::iterator ii = thcfg.selector.data.begin();
   while (ii != thcfg.selector.data.end()) {
@@ -582,7 +752,7 @@ char * thdb2d::get_projection_title(thdb2dprj * prj) {
   thdb_object_list_type::iterator obi = this->db->object_list.begin(), obi2;
   while (obi != this->db->object_list.end()) {
     if ((*obi)->get_class_id() == TT_SURVEY_CMD) {
-      ((thsurvey*)(*obi))->num1 = 0;
+      ((thsurvey*)(*obi).get())->num1 = 0;
     }
     obi++;
   }
@@ -590,18 +760,18 @@ char * thdb2d::get_projection_title(thdb2dprj * prj) {
   obi = this->db->object_list.begin();
   while (obi != this->db->object_list.end()) {
     if (((*obi)->get_class_id() == TT_SCRAP_CMD) &&
-        (((thscrap*)(*obi))->proj->id == prj->id) && 
-        (((thscrap*)(*obi))->exported)) {
-      if (((thscrap*)(*obi))->fsptr != NULL) {
-        ((thscrap*)(*obi))->fsptr->num1++;
-      } else if (((thscrap*)(*obi))->centerline_survey != NULL) {
-        ((thscrap*)(*obi))->centerline_survey->num1++;
+        (((thscrap*)(*obi).get())->proj->id == prj->id) && 
+        (((thscrap*)(*obi).get())->exported)) {
+      if (((thscrap*)(*obi).get())->fsptr != NULL) {
+        ((thscrap*)(*obi).get())->fsptr->num1++;
+      } else if (((thscrap*)(*obi).get())->centerline_survey != NULL) {
+        ((thscrap*)(*obi).get())->centerline_survey->num1++;
       } else {
         // prejde vsetky oznacene a da im num1 = 1
         obi2 = this->db->object_list.begin();
         while (obi2 != this->db->object_list.end()) {
           if (((*obi2)->get_class_id() == TT_SURVEY_CMD) && ((*obi2)->selected)) {
-            ((thsurvey*)(*obi2))->num1 = 1;
+            ((thsurvey*)(*obi2).get())->num1 = 1;
           }
           obi2++;
         }
