@@ -503,7 +503,7 @@ void thexpmap::export_kml(class thdb2dxm * maps, class thdb2dprj * prj)
     for(i = 0; i < nstat; i++) {
       station = &(db->db1d.station_vec[i]);
       if ((station->flags & TT_STATIONFLAG_ENTRANCE) != 0) {
-        thcs2cs(thcs_get_params(thcfg.outcs), thcs_get_params(TTCS_LONG_LAT),
+        thcs2cs(thcfg.outcs, TTCS_LONG_LAT,
           station->x, station->y, station->z, x, y, z);
         fprintf(out, "<Placemark>\n");
         fprintf(out, "<styleUrl>#ThEntranceIcon</styleUrl>");
@@ -571,13 +571,13 @@ void thexpmap::export_kml(class thdb2dxm * maps, class thdb2dprj * prj)
                   fprintf(out,"<innerBoundaryIs>\n");
                 fprintf(out,"<LinearRing>\n<coordinates>\n");
                 for(ip = it->m_point_list.begin(); ip != it->m_point_list.end(); ip++) {
-                  thcs2cs(thcs_get_params(thcfg.outcs), thcs_get_params(TTCS_LONG_LAT),
+                  thcs2cs(thcfg.outcs, TTCS_LONG_LAT,
                     ip->m_x, ip->m_y, scrap->z, x, y, z);
                   fprintf(out, "\t%.14f,%.14f,%.14f ", x / THPI * 180.0, y / THPI * 180.0, 0.0);
                 }
                 if (it->m_point_list.size() > 0) {
                 	ip = it->m_point_list.begin();
-                    thcs2cs(thcs_get_params(thcfg.outcs), thcs_get_params(TTCS_LONG_LAT),
+                    thcs2cs(thcfg.outcs, TTCS_LONG_LAT,
                       ip->m_x, ip->m_y, scrap->z, x, y, z);
                     fprintf(out, "\t%.14f,%.14f,%.14f ", x / THPI * 180.0, y / THPI * 180.0, 0.0);
                 }
@@ -670,19 +670,19 @@ void thexpmap::export_bbox(class thdb2dxm * maps, class thdb2dprj * prj)
             scrap = (thscrap*) cmi->object;
             if (!thisnan(scrap->lxmin)) {
 	    
-              thcs2cs(thcs_get_params(thcfg.outcs), thcs_get_params(TTCS_LONG_LAT),
+              thcs2cs(thcfg.outcs, TTCS_LONG_LAT,
                 scrap->lxmin + prj->rshift_x, scrap->lymin + prj->rshift_y, scrap->z + prj->rshift_z, cx, cy, cz);
               lim.Add(cx / THPI * 180.0, cy / THPI * 180.0, cz);
 
-              thcs2cs(thcs_get_params(thcfg.outcs), thcs_get_params(TTCS_LONG_LAT),
+              thcs2cs(thcfg.outcs, TTCS_LONG_LAT,
                 scrap->lxmin + prj->rshift_x, scrap->lymax + prj->rshift_y, scrap->z + prj->rshift_z, cx, cy, cz);
               lim.Add(cx / THPI * 180.0, cy / THPI * 180.0, cz);
 
-              thcs2cs(thcs_get_params(thcfg.outcs), thcs_get_params(TTCS_LONG_LAT),
+              thcs2cs(thcfg.outcs, TTCS_LONG_LAT,
                 scrap->lxmax + prj->rshift_x, scrap->lymin + prj->rshift_y, scrap->z + prj->rshift_z, cx, cy, cz);
               lim.Add(cx / THPI * 180.0, cy / THPI * 180.0, cz);
 
-              thcs2cs(thcs_get_params(thcfg.outcs), thcs_get_params(TTCS_LONG_LAT),
+              thcs2cs(thcfg.outcs, TTCS_LONG_LAT,
                 scrap->lxmax + prj->rshift_x, scrap->lymax + prj->rshift_y, scrap->z + prj->rshift_z, cx, cy, cz);
               lim.Add(cx / THPI * 180.0, cy / THPI * 180.0, cz);
 	      
