@@ -31,6 +31,7 @@
 #include "thdate.h"
 #include "thtfangle.h"
 #include "thinfnan.h"
+#include "thdata.h"
 
 thsurvey::thsurvey()
 {
@@ -41,7 +42,26 @@ thsurvey::thsurvey()
   this->decdef = false;
   this->decuds = thnan;
   this->num1 = 0;
+  this->level = 0;
   this->person_renames.clear();
+  this->entrance.clear();
+  this->data = NULL;
+  this->privatens = true;
+  this->surveyns = NULL;
+}
+
+
+thsurvey * thsurvey::get_nss()
+{
+  if (this->surveyns == NULL) {
+    this->surveyns = this;
+    if (!this->privatens) {
+      while ((!this->surveyns->privatens) && (this->surveyns->fsptr != NULL)) {
+        this->surveyns = this->surveyns->fsptr;
+      }
+    }
+  }
+  return this->surveyns;
 }
 
 
@@ -49,7 +69,6 @@ thsurvey::~thsurvey()
 {
 }
 
-
 int thsurvey::get_class_id() 
 {
   return TT_SURVEY_CMD;
@@ -70,21 +89,21 @@ int thsurvey::get_cmd_nargs()
 }
 
 
-char * thsurvey::get_cmd_end()
+const char * thsurvey::get_cmd_end()
 {
   // insert endcommand if multiline command
   return NULL;
 }
 
 
-char * thsurvey::get_cmd_name()
+const char * thsurvey::get_cmd_name()
 {
   // insert endcommand if multiline command
   return "survey";
 }
 
 
-thcmd_option_desc thsurvey::get_cmd_option_desc(char * opts)
+thcmd_option_desc thsurvey::get_cmd_option_desc(const char * opts)
 {
   int id = thmatch_token(opts, thtt_survey_opt);
   if (id == TT_SURVEY_UNKNOWN)
@@ -98,6 +117,20 @@ thcmd_option_desc thsurvey::get_cmd_option_desc(char * opts)
 }
 
 
+enum {
+  TT_SNS_EXPORT,
+  TT_SNS_PRIVATE,
+  TT_SNS_UNKNOWN,
+};
+ 
+static const thstok thtt_sns[] = {
+  {"off",TT_SNS_EXPORT},
+  {"on",TT_SNS_PRIVATE},
+	{NULL, TT_SNS_UNKNOWN},
+};
+
+
+
 void thsurvey::set(thcmd_option_desc cod, char ** args, int argenc, unsigned long indataline)
 {
   thperson tmpp1, tmpp2;
@@ -106,6 +139,23 @@ void thsurvey::set(thcmd_option_desc cod, char ** args, int argenc, unsigned lon
     
   switch (cod.id) {
 
+    case TT_SURVEY_NAMESPACE:
+      switch (thmatch_token(*args, thtt_sns)) {
+
+        case TT_SNS_PRIVATE:
+          this->privatens = false;
+          break;
+
+        case TT_SNS_EXPORT:
+          this->privatens = false;
+          break;
+
+        default:
+          ththrow("invalid -namespace switch -- {}", *args);
+
+      }
+      break;
+
     case TT_SURVEY_DECLINATION:
       this->parse_declination(*args);
       break;
@@ -117,12 +167,16 @@ void thsurvey::set(thcmd_option_desc cod, char ** args, int argenc, unsigned lon
       tmpp2.parse(this->db, this->db->buff_enc.get_buffer());
       this->person_renames[tmpp1] = tmpp2;
       break;
+
+    case TT_SURVEY_ENTRANCE:
+      thparse_objectname(this->entrance, & this->db->buff_stations, *args);
+      break;
       
     case TT_DATAOBJECT_NAME:
       if (th_is_keyword(*args))
         this->name = this->db->strstore(*args);
       else 
-        ththrow(("invalid keyword -- %s", *args));
+        ththrow("invalid keyword -- {}", *args);
       break;
       
     default:
@@ -132,12 +186,6 @@ void thsurvey::set(thcmd_option_desc cod, char ** args, int argenc, unsigned lon
 }
 
 
-void thsurvey::self_delete()
-{
-  delete this;
-}
-
-
 void thsurvey::self_print_properties(FILE * outf)
 {
   size_t nval, idx;
@@ -145,7 +193,7 @@ void thsurvey::self_print_properties(FILE * outf)
   thdataobject::self_print_properties(outf);
   fprintf(outf,"thsurvey:\n");
   fprintf(outf,"\tfull name: \"%s\" (\"%s\")\n", this->full_name,
-    this->reverse_full_name);
+    this->reverse_full_name.get_buffer());
   if (this->decdef) {
     fprintf(outf,"\tdeclination:\n");
     nval = this->declin.get_size();
@@ -165,7 +213,7 @@ int thsurvey::get_context()
   return (THCTX_SURVEY | THCTX_NONE);
 }
 
-char * thsurvey::get_full_name()
+const char * thsurvey::get_full_name()
 {
   return this->full_name;
 }
@@ -178,7 +226,7 @@ void thsurvey::parse_declination(char * str)
   thdate dd;
   double decl, //year, 
     dumdt;
-  double * all_data;
+  double * all_data = NULL;
   double data2 [2];
   int nid, idx, dateidx, typedt;
   
@@ -188,7 +236,7 @@ void thsurvey::parse_declination(char * str)
 
   // ak 1 4 6 ... error
   if ((nargs == 1) || (((nargs % 2) == 0) && (nargs > 3)))
-    ththrow(("invalid declination specification -- \"%s\"",str))
+    ththrow("invalid declination specification -- \"{}\"",str);
 
   // ak 0 tak resetuje a nastavi na 0
   if (nargs == 0) {
@@ -204,7 +252,7 @@ void thsurvey::parse_declination(char * str)
   if (nargs == 1) {
     thparse_double(nid, decl, args[0]);
     if (nid != TT_SV_NUMBER)
-      ththrow(("invalid declination -- %s", args[0]))
+      ththrow("invalid declination -- {}", args[0]);
     decl = du.transform(decl);
     data2[0] = 0.0;
     data2[1] = decl;
@@ -225,7 +273,7 @@ void thsurvey::parse_declination(char * str)
       thparse_double(nid, decl, args[2 * idx + 1]);
       if (nid != TT_SV_NUMBER) {
         delete [] all_data;
-        ththrow(("invalid declination -- %s", args[2 * idx + 1]))
+        ththrow("invalid declination -- {}", args[2 * idx + 1]);
       }
       this->decuds = decl;
     }
@@ -236,14 +284,14 @@ void thsurvey::parse_declination(char * str)
         dd.parse(args[2 * idx]);
       } catch (...) {
         delete [] all_data;
-        threthrow(("invalid declination specification"))
+        threthrow("invalid declination specification");
       }
       all_data[2 * dateidx] = dd.get_start_year();
       // parse declination
       thparse_double(nid, decl, args[2 * idx + 1]);
       if (nid != TT_SV_NUMBER) {
         delete [] all_data;
-        ththrow(("invalid declination -- %s", args[2 * idx + 1]))
+        ththrow("invalid declination -- {}", args[2 * idx + 1]);
       }
       all_data[2 * dateidx + 1] = du.transform(decl);
       dateidx++;
@@ -266,7 +314,8 @@ void thsurvey::parse_declination(char * str)
 void thsurvey::full_name_reverse()
 {
   size_t fnln = strlen(this->full_name), i1, i2, si; //, ei;
-  char * c1, * c2, *c3;
+  const char * c1, *c3;
+  char * c2;
   c1 = this->full_name;
   si = 0;
   bool has_dot = false;
@@ -274,7 +323,7 @@ void thsurvey::full_name_reverse()
     if ((*c1 == '.') || ((i1 == fnln) && has_dot)) {
       has_dot = true;
       c3 = this->full_name + si;
-      c2 = this->reverse_full_name + fnln - i1;
+      c2 = this->reverse_full_name.get_buffer() + fnln - i1;
       if (i1 < fnln) {
         c2--;
         *c2 = '.';
@@ -287,5 +336,29 @@ void thsurvey::full_name_reverse()
   }
 }
 
+thsurvey * thsurvey_get_entrance_fs(thsurvey * s) {
+	thdataobject * o;
+	o = s->foptr;
+	while (o != NULL) {
+		if (o->get_class_id() == TT_SURVEY_CMD) {
+			s = (thsurvey *)o;
+			if (!s->entrance.is_empty()) return s;
+			s = thsurvey_get_entrance_fs(s);
+			if (s != NULL) return s;
+		}
+		o = o->nsptr;
+	}
+	return NULL;
+}
+
+thobjectname thsurvey::get_entrance() {
+	thsurvey * s(this);
+	while ((s != NULL) and s->entrance.is_empty()) s = s->fsptr;
+	if (s == NULL) s = thsurvey_get_entrance_fs(this);
+	if (s == NULL) return thobjectname();
+	return s->entrance;
+}
+
+
 
 
